#include <EncoderButton.h>#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>

#define ms_per_hour  3600000
#define ms_per_min    60000
#define ms_per_sec    1000
  
  unsigned long now;
  unsigned long hour;
  unsigned long minute;
  unsigned long second;

int piezoPin = 5;

//                                         (clk, din, dc, ce, rst)
Adafruit_PCD8544 display = Adafruit_PCD8544(16,  15,  2,  0,  1);

EncoderButton eb1(7,8,9);

const int Relay1 = 20; 
const int Relay2 = 21;
unsigned long  CycleRunInterval = 1000;
unsigned long  CyclePauseInterval = 1000;
unsigned long  SoakInterval = 0;
unsigned long previousMillis = 0; 
unsigned long currentMillis = 0; 
int RelayState = LOW;
unsigned long  MachineState = 0; // 0 : select from menu type of program; 1: select soak time; 2 select wash time; 3 soaking; 4 running; 5 pause; 6 end;
unsigned long  WashTimer = 0;
unsigned long  SoakTimer = 0;
unsigned long  previousTimerMillis = 0;
unsigned long  SelectedWashProgram = 0;
long  MenuCounter = 0;
char *WashProgramMenu []= {"Strong", "Normal", "Gentle"};
int Paused = false;

//wash program events
void onWashProgramPress(EncoderButton& eb) {
  playtone();
  SelectedWashProgram = MenuCounter;
   switch (SelectedWashProgram) {
    case 0:
      // STRONG wash
      CycleRunInterval = 5000;
      CyclePauseInterval = 2000;
      break;
    case 1:
      // NORMAL wash
      CycleRunInterval = 3000;
      CyclePauseInterval = 2000;
      break;
    case 2:
      // GENTlE wash
      CycleRunInterval = 1000;
      CyclePauseInterval = 2000;
      break;
    default:
      CycleRunInterval = 1000;
      CyclePauseInterval = 1000;
  }
  
  Serial.println("SelectedWashProgram:");
  Serial.print("CycleRunInterval:");  Serial.println(CycleRunInterval);
  Serial.print("CyclePauseInterval:");  Serial.println(CyclePauseInterval);
  Serial.print("MenuCounter:");  Serial.println(MenuCounter);
  MachineState = 1;
  MenuCounter = 0;
}

void onWashProgramScroller(EncoderButton& eb) {
  playscroll();
  if (MenuCounter > 2) MenuCounter = 0;
  if (MenuCounter < 0) MenuCounter = 2;
  MenuCounter += eb.increment();
  if (MenuCounter > 2) MenuCounter = 0;
  if (MenuCounter < 0) MenuCounter = 2;
}




//wash timer events
void onWashTimePress(EncoderButton& eb) {
   playtone();
   WashTimer = (MenuCounter * 60000);
   
  Serial.println("Washtimer:");
  Serial.println(WashTimer);
  Serial.println("MenuCounter:");  Serial.println(MenuCounter);
   MachineState =3;
   MenuCounter = 0;
}
void onWashTimeScroller(EncoderButton& eb) {
   playscroll();
   MenuCounter += (eb.increment()*15);
   if(MenuCounter>60){MenuCounter = 60;}
   if(MenuCounter<15){MenuCounter = 15;}
}

//soak time
void onSoakTimePress(EncoderButton& eb) {
   playtone();
   SoakTimer = (MenuCounter * 60000);
   
   Serial.println("SoakTimer:");
   Serial.println(SoakTimer);
   Serial.println("MenuCounter:");  Serial.println(MenuCounter);
   MenuCounter = 15;
   MachineState = 2;
}
void onSoakTimeScroller(EncoderButton& eb) {
   playscroll();
   if(MenuCounter <= 0) {MenuCounter = 0;}
   if(MenuCounter > 600){MenuCounter = 600;}
   MenuCounter += (eb.increment()*15);
   if(MenuCounter <= 0){MenuCounter = 0;}
   if(MenuCounter > 600){MenuCounter = 600;}
}

void onButton1LongPress(EncoderButton& eb) {
   
      digitalWrite(Relay1, LOW);
      digitalWrite(Relay2, LOW);
   MachineState = 5;
}

void onDoNothing(EncoderButton& eb){}

void setup() {
  Serial.begin(9600);
  delay(500);
  Serial.println("Washing machine program");
  
  eb1.setClickHandler(onDoNothing);
  eb1.setEncoderHandler(onDoNothing);
  eb1.setLongPressHandler(onDoNothing);
  display.begin();
  display.setContrast(53);
  display.clearDisplay();
  pinMode(Relay1, OUTPUT);
  pinMode(Relay2, OUTPUT);
}

void loop() {
  display.setTextColor(BLACK);
  display.setCursor(0,0);
  display.clearDisplay();
  switch (MachineState) {
    case 0:
      // select wash strength
      eb1.setClickHandler(onWashProgramPress);
      eb1.setEncoderHandler(onWashProgramScroller);  
      eb1.setLongPressHandler(onDoNothing);
      display.setTextSize(1);
      display.println("Program \n Select:\n");
      display.setTextSize(2);
      display.println(WashProgramMenu[MenuCounter]);
      display.display();
      previousTimerMillis =0;
      break;
    case 1:
      // Select soak time
      eb1.setClickHandler(onSoakTimePress);
      eb1.setEncoderHandler(onSoakTimeScroller);
      eb1.setLongPressHandler(onDoNothing);
      display.setTextSize(1);
      display.println("Soak Time:\n\n");
      display.setTextSize(2);
      now = MenuCounter;
      hour = (now / 60);
      minute = now - (hour * 60);
      display.print(hour);display.print("h ");
      display.print(minute);display.print("m");
      display.display();
      break;
    case 2:
      // Select wash time
      eb1.setClickHandler(onWashTimePress);
      eb1.setEncoderHandler(onWashTimeScroller);
      eb1.setLongPressHandler(onDoNothing);
      display.setTextSize(1);
      display.println("Wash Time:\n\n");
      display.setTextSize(2);
      now = MenuCounter;
      display.print(MenuCounter);display.print("m");
      display.display();
      previousTimerMillis = 0;
      break;
    case 3:
      //Soaking
      eb1.setClickHandler(onDoNothing);
      eb1.setEncoderHandler(onDoNothing);
      eb1.setLongPressHandler(onButton1LongPress);

      //start timer for soaking
      if(previousTimerMillis <= 0){
          previousTimerMillis = millis();
      }
      
      if ((millis() - previousTimerMillis) >= SoakTimer) {
        MachineState = 4;
        previousTimerMillis = 0;
        SoakTimer = 0;
      }else{
        display.setTextSize(1);
        display.println("Soaking:\n\n");
        display.setTextSize(2);
        now = SoakTimer - (millis() - previousTimerMillis);
        hour = (now / ms_per_hour);
        minute = (now - (hour * ms_per_hour))/ms_per_min;
        second = (now - (minute*ms_per_min))/ms_per_sec;
        display.print(hour);display.print(":");
        if(minute < 10){display.print(0);}
        display.print(minute);display.print(":");
        if(second < 10){display.print(0);}
        display.print(second);
        display.display();
      }
      break;
    case 4:
      // washing
      eb1.setClickHandler(onDoNothing);
      eb1.setEncoderHandler(onDoNothing);
      eb1.setLongPressHandler(onButton1LongPress);
      
      
      //start timer for washing
      if(previousTimerMillis<=0){
          previousTimerMillis = millis();
      }
      
      if ((millis() - previousTimerMillis) >= WashTimer) {
        MachineState = 5; 
        previousTimerMillis = 0;
        WashTimer = 0;
      }else{
        display.setTextSize(1);
        display.println("Washing:\n\n");
        display.setTextSize(2);
        now = WashTimer - (millis() - previousTimerMillis);
        hour = (now / ms_per_hour);
        minute = (now - (hour * ms_per_hour))/ms_per_min;
        second = (now - (minute*ms_per_min))/ms_per_sec;
        if(minute < 10){display.print(0);}
        display.print(minute);display.print(":");
        if(second < 10){display.print(0);}
        display.print(second);
        display.display();
        runCycle();
      }
      break;
    case 5:
      //end of wash cycle
      eb1.setClickHandler(onDoNothing);
      eb1.setEncoderHandler(onDoNothing);
      eb1.setLongPressHandler(onDoNothing);
      WashTimer = 0;
      SoakTimer = 0;
      MenuCounter = 0;
      SelectedWashProgram = 0;
      digitalWrite(Relay1, LOW);
      digitalWrite(Relay2, LOW);
      display.setTextSize(2);
      display.println("Ta Dah!");
      display.display();
      MachineState =0;
      playtada();
      break;
    default:
    ;
  }
  eb1.update();
}


void runCycle(){
  if (millis() - previousMillis <= (CycleRunInterval + CyclePauseInterval)){
    Serial.println("Washing Cycle:");
    if(millis() - previousMillis <= CycleRunInterval){
      //run
    Serial.print("Run:");

      if(RelayState){
        Serial.println(">>>>>");
      }else{
        Serial.println("<<<<<");
      }
      digitalWrite(Relay1, !RelayState);
      digitalWrite(Relay2, RelayState);
    }else{
      //pause
    Serial.println("Pause");
      digitalWrite(Relay1, LOW);
      digitalWrite(Relay2, LOW);
    } 
  }else{
      RelayState = !RelayState; // direction change
      Serial.print("RelayState:");
      Serial.println(RelayState);
      previousMillis = millis(); //reset timer
  }
}

void playtone(){
  tone(piezoPin, 500, 50);
}
void playscroll(){
  tone(piezoPin, 1000, 5);
}

void playtada(){
  tone(piezoPin, 1000, 250);
  delay(500);
  tone(piezoPin, 1000, 550);
  delay(3000);
}
